# -*- coding: utf-8 -*-
#
# django-codenerix-email
#
# Codenerix GNU
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.urls import re_path
from codenerix_email.views import (
    EmailTemplateList,
    EmailTemplateCreate,
    EmailTemplateCreateModal,
    EmailTemplateUpdate,
    EmailTemplateUpdateModal,
    EmailTemplateDelete,
    EmailMessageList,
    EmailMessageCreate,
    EmailMessageCreateModal,
    EmailMessageUpdate,
    EmailMessageUpdateModal,
    EmailMessageDelete,
    EmailMessageDetails,
    EmailMessageDetailsModal,
    EmailReceivedList,
    EmailReceivedDetails,
    EmailReceivedDetailsModal,
    EmailReceivedSubList,
)


urlpatterns = [
    re_path(
        r"^emailtemplates$",
        EmailTemplateList.as_view(),
        name="CDNX_emails_emailtemplates_list",
    ),
    re_path(
        r"^emailtemplates/add$",
        EmailTemplateCreate.as_view(),
        name="CDNX_emails_emailtemplates_add",
    ),
    re_path(
        r"^emailtemplates/addmodal$",
        EmailTemplateCreateModal.as_view(),
        name="CDNX_emails_emailtemplates_addmodal",
    ),
    re_path(
        r"^emailtemplates/(?P<pk>\w+)/edit$",
        EmailTemplateUpdate.as_view(),
        name="CDNX_emails_emailtemplates_edit",
    ),
    re_path(
        r"^emailtemplates/(?P<pk>\w+)/editmodal$",
        EmailTemplateUpdateModal.as_view(),
        name="CDNX_emails_emailtemplates_editmodal",
    ),
    re_path(
        r"^emailtemplates/(?P<pk>\w+)/delete$",
        EmailTemplateDelete.as_view(),
        name="CDNX_emails_emailtemplates_delete",
    ),
    re_path(
        r"^emailmessages$",
        EmailMessageList.as_view(),
        name="CDNX_emails_emailmessages_list",
    ),
    re_path(
        r"^emailmessages/add$",
        EmailMessageCreate.as_view(),
        name="CDNX_emails_emailmessages_add",
    ),
    re_path(
        r"^emailmessages/addmodal$",
        EmailMessageCreateModal.as_view(),
        name="CDNX_emails_emailmessages_addmodal",
    ),
    re_path(
        r"^emailmessages/(?P<pk>\w+)$",
        EmailMessageDetails.as_view(),
        name="CDNX_emails_emailmessages_details",
    ),
    re_path(
        r"^emailmessages/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/modal$",
        EmailMessageDetailsModal.as_view(),
        name="CDNX_emails_emailmessages_sublist_details_modal",
    ),
    re_path(
        r"^emailmessages/(?P<pk>\w+)/edit$",
        EmailMessageUpdate.as_view(),
        name="CDNX_emails_emailmessages_edit",
    ),
    re_path(
        r"^emailmessages/(?P<pk>\w+)/editmodal$",
        EmailMessageUpdateModal.as_view(),
        name="CDNX_emails_emailmessages_editmodal",
    ),
    re_path(
        r"^emailmessages/(?P<pk>\w+)/delete$",
        EmailMessageDelete.as_view(),
        name="CDNX_emails_emailmessages_delete",
    ),
    re_path(
        r"^emailreceiveds$",
        EmailReceivedList.as_view(),
        name="CDNX_emails_emailreceiveds_list",
    ),
    re_path(
        r"^emailreceiveds/(?P<pk>\w+)$",
        EmailReceivedDetails.as_view(),
        name="CDNX_emails_emailreceiveds_details",
    ),
    re_path(
        r"^emailreceiveds/(?P<pk>\w+)/sublist$",
        EmailReceivedSubList.as_view(),
        name="CDNX_emails_emailreceiveds_sublist",
    ),
    re_path(
        r"^emailreceiveds/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/modal$",
        EmailReceivedDetailsModal.as_view(),
        name="CDNX_emails_emailreceiveds_sublist_details_modal",
    ),
]



# Generated by Django 5.0.6 on 2024-06-04 11:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("codenerix_email", "0010_emailmessage_content_subtype_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="emailmessage",
            name="content_subtype",
            field=models.CharField(
                choices=[("plain", "Plain"), ("html", "HTML Web")],
                default="html",
                max_length=256,
                verbose_name="Content Subtype",
            ),
        ),
        migrations.AlterField(
            model_name="emailtemplate",
            name="content_subtype",
            field=models.CharField(
                choices=[("plain", "Plain"), ("html", "HTML Web")],
                default="html",
                max_length=256,
                verbose_name="Content Subtype",
            ),
        ),
    ]



# Generated by Django 5.0.4 on 2024-05-22 11:41

import uuid
from django.db import migrations, models


def create_uuid(apps, schema_editor):
    EmailMessage = apps.get_model("codenerix_email", "EmailMessage")
    for email_message in EmailMessage.objects.all():
        email_message.uuid = uuid.uuid4()
        email_message.save()


class Migration(migrations.Migration):

    dependencies = [
        ("codenerix_email", "0008_auto_20171201_0928"),
    ]

    operations = [
        migrations.AddField(
            model_name="emailmessage",
            name="opened",
            field=models.DateTimeField(
                blank=True, default=None, null=True, verbose_name="Opened"
            ),
        ),
        migrations.AddField(
            model_name="emailmessage",
            name="uuid",
            field=models.UUIDField(blank=True, null=True),
        ),
        migrations.RunPython(create_uuid),
        migrations.AlterField(
            model_name="emailmessage",
            name="uuid",
            field=models.UUIDField(
                default=uuid.uuid4,
                editable=False,
                unique=True,
                verbose_name="UUID",
            ),
        ),
    ]



import re

import logging

from django.conf import settings
from django.core.management.base import BaseCommand, CommandError
from zoneinfo import ZoneInfo

from email import message_from_bytes
from email.header import decode_header
from email.message import Message
from email.parser import HeaderParser
from typing import Optional

from codenerix_email.models import (
    EmailMessage,
    EmailReceived,
    BOUNCE_SOFT,
    BOUNCE_HARD,
)


# Silence DEBUG logs from imapclient
logging.getLogger("imapclient").setLevel(logging.WARNING)

import imaplib  # noqa: E402
from imapclient import IMAPClient  # noqa: E402
from imapclient.exceptions import LoginError  # noqa: E402


class Command(BaseCommand):
    help = "Fetches new emails from the configured IMAP account."

    def add_arguments(self, parser):
        # Named (optional) arguments
        parser.add_argument(
            "--silent",
            action="store_true",
            dest="silent",
            default=False,
            help="Enable silent mode",
        )

        parser.add_argument(
            "--tracking-id", type=str, help="Tracking ID to filter"
        )
        parser.add_argument("--imap-id", type=str, help="IMAP ID to filter")
        parser.add_argument(
            "--message-id", type=str, help="Message-ID to filter"
        )
        parser.add_argument(
            "--all", action="store_true", help="Process all emails"
        )
        parser.add_argument(
            "--rewrite", action="store_true", help="Rewrite existing"
        )

    def handle(self, *args, **options):
        # Get configuration
        self.silent = options["silent"]
        self.verbose = not self.silent
        self.imap_id = options.get("imap_id")
        self.message_id = options.get("message_id")
        self.tracking_id = options.get("tracking_id")
        self.rewrite = options.get("rewrite", False)
        self.process_all = options.get("all", False)

        # Show header
        if self.verbose:
            self.stdout.write(
                self.style.SUCCESS("Starting IMAP email synchronization...")
            )

        # Get configuration from settings
        host = getattr(settings, "IMAP_EMAIL_HOST", None)
        port = getattr(settings, "IMAP_EMAIL_PORT", 993)
        user = getattr(settings, "IMAP_EMAIL_USER", None)
        password = getattr(settings, "IMAP_EMAIL_PASSWORD", None)
        ssl = getattr(settings, "IMAP_EMAIL_SSL", True)
        folder = getattr(settings, "IMAP_EMAIL_INBOX_FOLDER", "INBOX")

        # Verify that IMAP settings are configured
        if host is not None and port:
            # Validate configuration
            if user is None or password is None:
                if self.silent:
                    return
                else:
                    raise CommandError(
                        "IMAP user or password not configured. Please set "
                        "IMAP_EMAIL_USER and IMAP_EMAIL_PASSWORD in settings."
                    )

            try:
                # Connect to the IMAP server
                server = IMAPClient(host, port=port, ssl=ssl)
            except Exception as e:
                raise CommandError(
                    f"Failed to connect to IMAP server ("
                    f"{host=}, "
                    f"{port=}, "
                    f"ssl={ssl and 'yes' or 'no'}"
                    f"): {e}"
                ) from e

            try:
                # Login and select the inbox
                try:
                    server.login(user, password)
                except LoginError as e:
                    raise CommandError(
                        f"Failed to login to IMAP server with {user=}: {e}"
                    ) from e

                if folder:
                    try:
                        server.select_folder(folder, readonly=False)
                    except imaplib.IMAP4.error:
                        raise CommandError(f"Failed to select inbox {folder=}")

                # Process emails
                (created_count, overwritten_count) = self.process(server)
                count = created_count + overwritten_count

                # Show summary
                if self.verbose:
                    self.stdout.write(
                        self.style.SUCCESS(
                            f"Successfully synchronized {count} emails "
                            f"(new: {created_count}, "
                            f"overwritten: {overwritten_count})"
                        )
                    )

            except Exception as e:
                raise
                self.stderr.write(
                    self.style.ERROR(
                        f"An error occurred during synchronization: {e}"
                    )
                )

            finally:
                # Logout from the server
                try:
                    server.logout()
                except Exception:
                    pass

        elif self.verbose:
            raise CommandError(
                "IMAP settings not configured. Please set IMAP_EMAIL_HOST "
                "and IMAP_EMAIL_PORT in settings."
            )

    def process(self, server):
        """
        Connects to the IMAP server and fetches new emails,
        saving them as ReceivedEmail objects.
        """

        # Processed emails count
        created_count = 0
        overwrite_count = 0

        # Look up for emails
        if self.imap_id:
            # Search by specific IMAP ID
            try:
                imap_id = int(self.imap_id)
            except ValueError:
                raise CommandError(
                    f"Invalid IMAP ID '{self.imap_id}'. Must be an integer."
                )
            messages_ids = [imap_id]
            if self.verbose:
                self.stdout.write(
                    self.style.SUCCESS(
                        f"Processing email with IMAP ID {self.imap_id}."
                    )
                )

        elif self.message_id:
            # Search by specific Message-ID
            messages_ids = server.search(
                ["HEADER", "Message-ID", self.message_id]
            )
            if self.verbose:
                self.stdout.write(
                    self.style.SUCCESS(
                        f"Found {len(messages_ids)} email(s) with "
                        f"Message-ID {self.message_id}."
                    )
                )

        elif self.process_all:
            # Process all emails
            messages_ids = server.search(["ALL"])
            if self.verbose:
                self.stdout.write(
                    self.style.SUCCESS(
                        f"Found {len(messages_ids)} email(s) to process."
                    )
                )

        else:
            # Search by UNSEEN
            messages_ids = server.search(["UNSEEN"])
            if self.verbose:
                self.stdout.write(
                    self.style.SUCCESS(
                        f"Found {len(messages_ids)} new email(s) to process."
                    )
                )

        # If there are new messages, fetch and process them
        if messages_ids:
            # Fetch the full message and internal date
            fetched_data = server.fetch(
                messages_ids, ["BODY.PEEK[]", "INTERNALDATE"]
            )

            # Get the envelope (metadata) and the full body
            # Use IMAP IDs so identifiers do not change between sessions
            for imap_id, message_data in fetched_data.items():
                # Filter out by IMAP ID if specified
                if self.imap_id and str(imap_id) != self.imap_id:
                    continue

                # Get the raw email and internal date
                raw_email = message_data[b"BODY[]"]
                internal_date_naive = message_data[b"INTERNALDATE"]
                internal_date = internal_date_naive.replace(
                    tzinfo=ZoneInfo(settings.TIME_ZONE)
                )

                # Parse the email
                msg = message_from_bytes(raw_email)

                # Extract subject, efrom, eto & eid
                subject, encoding = decode_header(msg["Subject"])[0]
                if isinstance(subject, bytes):
                    subject = subject.decode(encoding or "utf-8")
                efrom = msg.get("From")
                eto = msg.get("To")
                eid = msg.get("Message-ID")

                # If we can't get a Message-ID, use the IMAP ID as fallback
                # to avoid duplicates
                if not eid:
                    eid = f"<imapid-{imap_id}@{settings.IMAP_EMAIL_HOST}>"

                # Avoid processing duplicates
                email_received = EmailReceived.objects.filter(eid=eid).first()
                if self.rewrite or not email_received:
                    # Process multipart emails
                    body_plain = ""
                    body_html = ""
                    if msg.is_multipart():
                        for part in msg.walk():
                            content_type = part.get_content_type()
                            if content_type == "text/plain" and not body_plain:
                                body_plain = part.get_payload(
                                    decode=True
                                ).decode(
                                    part.get_content_charset() or "utf-8",
                                    "ignore",
                                )
                            elif content_type == "text/html" and not body_html:
                                body_html = part.get_payload(
                                    decode=True
                                ).decode(
                                    part.get_content_charset() or "utf-8",
                                    "ignore",
                                )
                    else:
                        body_plain = msg.get_payload(decode=True).decode(
                            msg.get_content_charset() or "utf-8",
                            "ignore",
                        )

                    # Logic to associate replies/bounces with sent emails
                    try:
                        email_message = None

                        # Locate the tracking ID
                        tracking_id = self.find_tracking_id(msg)

                        # Filter out by tracking ID if specified
                        if (
                            self.tracking_id
                            and tracking_id != self.tracking_id
                        ):
                            continue

                        # If found, try to link to the sent email
                        if tracking_id:
                            try:
                                email_message = EmailMessage.objects.get(
                                    uuid=tracking_id
                                )
                            except EmailMessage.DoesNotExist:
                                email_message = None
                                if self.verbose:
                                    self.stdout.write(
                                        self.style.WARNING(
                                            f"Tracking ID {tracking_id} found "
                                            "but no matching sent email."
                                        )
                                    )

                    except Exception as e:
                        raise CommandError(
                            "Error while linking email with IMAP ID "
                            f"{imap_id} to sent email: {e}"
                        ) from e

                    # Heuristic keywords commonly found in bounce messages
                    (bounce_type, bounce_reason) = self.analyze_bounce(msg)

                    # Extract all headers into a dictionary
                    headers = {}
                    for header, value in msg.items():
                        decoded_value, encoding = decode_header(value)[0]
                        if isinstance(decoded_value, bytes):
                            decoded_value = decoded_value.decode(
                                encoding or "utf-8", "ignore"
                            )
                        headers[header] = decoded_value

                    # Create EmailReceived object if doesn't exist
                    if not email_received:
                        overwriting = False
                        email_received = EmailReceived()
                    else:
                        overwriting = True

                    # Populate fields
                    email_received.imap_id = imap_id
                    email_received.eid = eid
                    email_received.efrom = efrom
                    email_received.eto = eto
                    email_received.subject = subject
                    email_received.headers = headers
                    email_received.body_text = body_plain
                    email_received.body_html = body_html
                    email_received.date_received = internal_date
                    email_received.email = email_message
                    email_received.bounce_type = bounce_type
                    email_received.bounce_reason = bounce_reason

                    # Save the received email
                    email_received.save()

                    # Count created or overwritten
                    if overwriting:
                        overwrite_count += 1
                        verb = "Overwritten"
                    else:
                        created_count += 1
                        verb = "Created"

                    if self.verbose:
                        msg = (
                            f"{verb} email with IMAP ID: "
                            f"{imap_id} (link={tracking_id})"
                        )
                        if bounce_type:
                            bounce_type_str = (
                                bounce_type == BOUNCE_HARD and "Hard" or "Soft"
                            )
                            bounce_reason_str = bounce_reason or "Unknown"
                            self.stdout.write(
                                self.style.WARNING(
                                    f"{msg} "
                                    f"[{bounce_type_str} bounce, "
                                    f"reason={bounce_reason_str}]"
                                )
                            )
                        else:
                            self.stdout.write(self.style.SUCCESS(msg))

                else:
                    if self.verbose:
                        self.stdout.write(
                            self.style.WARNING(
                                f"Skipping email with IMAP ID: {imap_id} (DUP)"
                            )
                        )

                # Mark the message as read
                # (flag \Seen) avoid reprocessing
                server.add_flags(imap_id, [b"\\Seen"])

        return (created_count, overwrite_count)

    def find_tracking_id(self, msg: Message) -> str | None:
        """
        Searches for the X-Codenerix-Tracking-ID robustly in an email.

        It performs the search in three steps:
        1. In the main headers of the email.
        2. In the attached parts that are a complete email (message/rfc822).
        3. As a last resort, searches the text in the body of the message.
        """

        # Method 1: Search in main headers (for direct replies)
        tracking_id = msg.get("X-Codenerix-Tracking-ID", None)

        # Method 2: Search in attached parts (for bounces and forwards)
        if not tracking_id:
            # Not found directly in headers
            # Search in attached parts (for bounces and forwards)
            if msg.is_multipart():
                # Iterate through parts
                for part in msg.walk():
                    # Get the content type of the part
                    content_type = part.get_content_type()

                    # We look for an attachment that is itself an email
                    if content_type == "message/rfc822":
                        # The payload of this part is the original email
                        # The payload is a list of messages, take the first one
                        original_msg_payload = part.get_payload()
                        if (
                            isinstance(original_msg_payload, list)
                            and original_msg_payload
                        ):
                            original_msg = original_msg_payload[0]
                            if isinstance(original_msg, Message):
                                tracking_id = original_msg.get(
                                    "X-Codenerix-Tracking-ID"
                                )

                    elif content_type == "text/rfc822-headers":
                        # The payload is the raw headers of the original email
                        headers_payload = part.get_payload(decode=True)
                        if isinstance(headers_payload, bytes):
                            # Decode using the specified charset
                            charset = part.get_content_charset() or "utf-8"
                            headers_text = headers_payload.decode(
                                charset, errors="ignore"
                            )

                            # Parse headers text into a Message object
                            headers_msg = HeaderParser().parsestr(headers_text)
                            tracking_id = headers_msg.get(
                                "X-Codenerix-Tracking-ID"
                            )

            # Method 3: Search in the body text (fallback)
            if not tracking_id:
                # The original email might be quoted as plain text.
                body_text = ""
                if msg.is_multipart():
                    # Concatenate all text/plain parts
                    for part in msg.walk():
                        # We only want text/plain parts
                        if part.get_content_type() == "text/plain":
                            # Get the decoded payload
                            payload = part.get_payload(decode=True)
                            if isinstance(payload, bytes):
                                # Decode using the specified charset
                                charset = part.get_content_charset() or "utf-8"
                                body_text += payload.decode(
                                    charset, errors="ignore"
                                )
                else:
                    # Single part email, check if it's text/plain
                    if msg.get_content_type() == "text/plain":
                        # Get the decoded payload
                        payload = msg.get_payload(decode=True)
                        if isinstance(payload, bytes):
                            # Decode using the specified charset
                            charset = msg.get_content_charset() or "utf-8"
                            body_text = payload.decode(
                                charset, errors="ignore"
                            )

                # If we have body text, search for the header using regex
                if body_text:
                    # We use a regex to find the header in the text
                    match = re.search(
                        r"X-Codenerix-Tracking-ID:\s*([a-fA-F0-9\-]{36})",
                        body_text,
                    )

                    if match:
                        # If found, extract the tracking ID
                        tracking_id = match.group(1).strip()

        # Return the found tracking ID if any
        return tracking_id

    def analyze_bounce(
        self, msg: Message
    ) -> tuple[Optional[str], Optional[str]]:
        """
        Analyzes an email to determine if it is a bounce and of what type.

        Returns:
            A tuple (bounce_type, smtp_code).
            - bounce_type: BOUNCE_HARD, BOUNCE_SOFT, or None if not a bounce.
            - bounce_reason: the SMTP status code (e.g., '5.1.1') or None.
        """

        # Initialize
        bounce_type: Optional[str] = None
        bounce_reason: Optional[str] = None

        # Method 1: Look for DSN reports
        if (
            msg.get_content_type() == "multipart/report"
            and msg.get_param("report-type") == "delivery-status"
        ):
            # Iterate through parts to find the delivery-status part
            for part in msg.walk():
                # We look for the delivery-status part
                if part.get_content_type() == "message/delivery-status":
                    # The payload is a list of headers
                    payload = part.get_payload()
                    if payload and isinstance(payload, list):
                        # The first part contains the status headers
                        status_headers = payload[0]
                        if isinstance(status_headers, Message):
                            # Extract Action and Status headers
                            action = status_headers.get("Action", "").lower()
                            status_code = status_headers.get("Status", "")

                            # Check if action indicates failure
                            if action == "failed":
                                # Determine Hard/Soft by SMTP code (RFC3463)
                                if status_code.startswith("5."):
                                    # 5.x.x: permanent failure (hard)
                                    bounce_type = BOUNCE_HARD
                                    bounce_reason = status_code
                                    break
                                elif status_code.startswith("4."):
                                    # 4.x.x: temporary failure (soft)
                                    bounce_type = BOUNCE_SOFT
                                    bounce_reason = status_code
                                    break
                                else:
                                    # Unknown status, assume hard bounce
                                    bounce_type = BOUNCE_HARD
                                    bounce_reason = status_code or "Unknown"
                                    break

        # Method 2: Some mail servers include headers indicating a bounce
        if not bounce_type:
            if msg.get("X-Failed-Recipients"):
                # Presence of this header usually indicates a hard bounce
                bounce_type = BOUNCE_HARD
                bounce_reason = "Unknown (X-Failed-Recipients)"

            else:
                # Check for Auto-Submitted header
                if msg.get("Auto-Submitted", "").lower() in (
                    "auto-replied",
                    "auto-generated",
                ):
                    # It could be a bounce, but also an "Out of Office",
                    # so we combine it with a keyword search.
                    subject = msg.get("Subject", "").lower()
                    bounce_keywords = [
                        "undeliverable",
                        "delivery failed",
                        "failure notice",
                    ]

                    # If we find bounce keywords in the subject
                    if any(keyword in subject for keyword in bounce_keywords):
                        # Assume is a hard bounce
                        bounce_type = BOUNCE_HARD
                        bounce_reason = "Unknown (Auto-Submitted + Keyword)"

        # Method 3: keyword search (less reliable)
        if not bounce_type:
            # We look for common bounce keywords in the From or Subject headers
            # We avoid false positives by requiring specific keywords.
            from_header = msg.get("From", "").lower()
            subject_header = msg.get("Subject", "").lower()

            if "mailer-daemon@" in from_header or "postmaster@" in from_header:
                # Common bounce sender addresses
                bounce_type = BOUNCE_HARD
                bounce_reason = "Unknown (From Keyword)"
            else:
                # Check subject for common bounce keywords
                bounce_keywords = [
                    "undelivered",
                    "delivery error",
                    "mail delivery failed",
                ]
                if any(
                    keyword in subject_header for keyword in bounce_keywords
                ):
                    bounce_type = BOUNCE_HARD
                    bounce_reason = "Unknown (Subject Keyword)"

        return (bounce_type, bounce_reason)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-05-02 10:43
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('codenerix_email', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='EmailTemplateTextEN',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('subject', models.TextField(blank=True, verbose_name='Subject')),
                ('body', models.TextField(blank=True, verbose_name='Body')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='EmailTemplateTextES',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('subject', models.TextField(blank=True, verbose_name='Subject')),
                ('body', models.TextField(blank=True, verbose_name='Body')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.RemoveField(
            model_name='emailtemplate',
            name='body',
        ),
        migrations.RemoveField(
            model_name='emailtemplate',
            name='subject',
        ),
        migrations.AddField(
            model_name='emailtemplatetextes',
            name='email_template',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='es', to='codenerix_email.EmailTemplate'),
        ),
        migrations.AddField(
            model_name='emailtemplatetexten',
            name='email_template',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='en', to='codenerix_email.EmailTemplate'),
        ),
    ]



# Generated by Django 5.2.6 on 2025-09-24 09:21

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("codenerix_email", "0013_emailmessage_headers"),
    ]

    operations = [
        migrations.CreateModel(
            name="EmailReceived",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(auto_now_add=True, verbose_name="Created"),
                ),
                (
                    "updated",
                    models.DateTimeField(auto_now=True, verbose_name="Updated"),
                ),
                ("imap_id", models.IntegerField(default=0, verbose_name="IMAP ID")),
                (
                    "eid",
                    models.CharField(
                        help_text="Unique Message-ID header",
                        max_length=512,
                        unique=True,
                        verbose_name="Message ID",
                    ),
                ),
                ("efrom", models.EmailField(max_length=254, verbose_name="From")),
                ("eto", models.EmailField(max_length=254, verbose_name="To")),
                ("subject", models.CharField(max_length=256, verbose_name="Subject")),
                (
                    "headers",
                    models.JSONField(blank=True, null=True, verbose_name="Headers"),
                ),
                ("body_text", models.TextField(blank=True, verbose_name="Body (Text)")),
                ("body_html", models.TextField(blank=True, verbose_name="Body (HTML)")),
                (
                    "date_received",
                    models.DateTimeField(
                        auto_now_add=True, verbose_name="Date received"
                    ),
                ),
                (
                    "bounce_type",
                    models.CharField(
                        blank=True,
                        choices=[("S", "Soft"), ("H", "Hard")],
                        default=None,
                        max_length=1,
                        null=True,
                        verbose_name="Bounce Type",
                    ),
                ),
                (
                    "bounce_reason",
                    models.CharField(
                        blank=True,
                        default=None,
                        max_length=512,
                        null=True,
                        verbose_name="Bounce Reason",
                    ),
                ),
                (
                    "email",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="receiveds",
                        to="codenerix_email.emailmessage",
                    ),
                ),
            ],
            options={
                "abstract": False,
                "default_permissions": ("add", "change", "delete", "view", "list"),
            },
        ),
    ]



# -*- coding: utf-8 -*-
#
# django-codenerix-email
#
# Codenerix GNU
#
# Project URL : http://www.codenerix.com
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import base64

from typing import Optional, Tuple, List, Dict
from django.http import HttpResponse
from django.views.generic import View

from django.shortcuts import get_object_or_404
from django.utils.translation import gettext as _
from django.conf import settings
from django.http import HttpRequest, Http404
from django.db.models import Q, Count

from codenerix.multiforms import MultiForm  # type: ignore
from codenerix.views import (  # type: ignore
    GenList,
    GenCreate,
    GenCreateModal,
    GenUpdate,
    GenUpdateModal,
    GenDelete,
    GenDetail,
    GenDetailModal,
    SearchFilters,
)
from codenerix_email.models import (
    EmailTemplate,
    EmailMessage,
    EmailReceived,
    MODELS,
    BOUNCE_SOFT,
    BOUNCE_HARD,
)
from codenerix_email.forms import (
    EmailTemplateForm,
    EmailMessageForm,
    EmailReceivedForm,
)

formsfull: Dict[
    str,
    List[
        Tuple[
            Optional[HttpRequest],
            Optional[str],
            Optional[str],
        ]
    ],
] = {}

for info in MODELS:
    field = info[0]
    model = info[1]
    formsfull[model] = [(None, None, None)]
    for lang_code in settings.LANGUAGES_DATABASES:
        query = "from codenerix_email.models import {}Text{}\n".format(
            model, lang_code
        )
        query += "from codenerix_email.forms import {}TextForm{}".format(
            model, lang_code
        )
        exec(query)

        formsfull[model].append(
            (
                eval("{}TextForm{}".format(model, lang_code.upper())),
                field,
                None,
            )
        )


# ############################################
# EmailFollow
class EmailFollow(View):
    def get(self, request, *args, **kwargs):
        # Get uuid from email
        uid = kwargs.get("uuid_ext", None)

        if uid:
            # Get email message or return 404
            email_message = get_object_or_404(EmailMessage, uuid=uid)

            # Set email message as opened
            email_message.set_opened()

            # Return an image of 1x1 pixel
            return HttpResponse(
                base64.b64decode(
                    "R0lGODlhAQABAIAAAP///wAAACH5BAEAAAAALAAAAAABAAEAAAICRAEAOw=="
                ),
                content_type="image/gif",
            )

        else:
            # Return 404
            raise Http404


# ############################################
# EmailTemplate
class EmailTemplateList(GenList):
    model = EmailTemplate
    default_ordering = ["-created"]
    extra_context = {
        "menu": ["codenerix_email", "emailtemplate"],
        "bread": [_("Emails"), _("Email Template")],
    }

    def __fields__(self, info):
        fields = []
        fields.append(("pk", _("PK"), 100))
        fields.append(("cid", _("CID"), 100))
        fields.append(("efrom", _("From"), 100))
        fields.append(
            (
                f"{self.language}__subject",
                _("Subject"),
                100,
                None,
                "shorttext:150",
            )
        )
        fields.append(("content_subtype", _("Content Subtype"), 100))
        return fields


class EmailTemplateCreate(MultiForm, GenCreate):
    model = EmailTemplate
    form_class = EmailTemplateForm
    forms = formsfull["EmailTemplate"]


class EmailTemplateCreateModal(GenCreateModal, EmailTemplateCreate):
    pass


class EmailTemplateUpdate(MultiForm, GenUpdate):
    model = EmailTemplate
    form_class = EmailTemplateForm
    forms = formsfull["EmailTemplate"]


class EmailTemplateUpdateModal(GenUpdateModal, EmailTemplateUpdate):
    pass


class EmailTemplateDelete(GenDelete):
    model = EmailTemplate


# ############################################
# EmailMessage
class EmailMessageList(GenList):
    model = EmailMessage
    show_details = True
    search_filter_button = True
    datetime_filter = "updated"
    default_ordering = ["-created"]
    gentranslate = {
        "sending": _("Sending"),
        "sent": _("Sent"),
        "notsent": _("Not sent!"),
        "waiting": _("Waiting"),
    }
    extra_context = {
        "menu": ["codenerix_email", "emailmessages"],
        "bread": [_("Emails"), _("Email Messages")],
    }
    annotations = {
        # "bounces_soft_count": Count(
        #     "receiveds__pk", filter=Q(receiveds__bounce_type=BOUNCE_SOFT)
        # ),
        # "bounces_hard_count": Count(
        #     "receiveds__pk", filter=Q(receiveds__bounce_type=BOUNCE_HARD)
        # ),
        "bounces_total_count": Count(
            "receiveds__pk", filter=Q(receiveds__bounce_type__isnull=False)
        ),
    }

    def __fields__(self, info):
        fields = []
        fields.append(("sending", None))
        fields.append(("error", None))
        fields.append(("sent", _("Send")))
        fields.append(("priority", _("Priority")))
        fields.append(("created", _("Created")))
        fields.append(("updated", _("Updated")))
        fields.append(("opened", _("Opened")))
        # fields.append(("efrom", _("From")))
        fields.append(("eto", _("To")))
        fields.append(("subject", _("Subject")))
        fields.append(("bounces_total_count", _("Bounces")))
        # fields.append(("bounces_soft_count", _("Soft bounces")))
        # fields.append(("bounces_hard_count", _("Hard bounces")))
        fields.append(("retries", _("Retries")))
        fields.append(("next_retry", _("Next retry")))
        fields.append(("pk", _("ID")))
        fields.append(("uuid", _("UUID")))
        fields.append(("unsubscribe_url", _("Unsubscribe")))
        fields.append(("content_subtype", _("Content Subtype")))
        return fields

    def __searchF__(self, info):  # noqa: N802
        def mailstatus(x):
            if x == "D":
                return Q(error=False, sent=True)
            elif x == "P":
                return Q(error=False, sent=False, sending=False)
            elif x == "S":
                return Q(error=False, sent=False, sending=True)
            elif x == "E":
                return Q(error=True)
            else:
                return Q()

        mailoptions = [
            ("D", _("Sent")),  # Sent - Done
            ("P", _("Pending")),  # Pending - Pending
            ("S", _("Sending")),  # Sending - Sending
            ("E", _("Error")),  # Error - Error
        ]

        return {
            "sent": (_("Sent"), lambda x: mailstatus(x), mailoptions),
            "uuid": (_("UUID"), lambda x: Q(uuid__icontains=x), "input"),
            "priority": (_("Priority"), lambda x: Q(priority=x), "input"),
            "opened": (
                _("Opened"),
                lambda x: ~Q(opened__isnull=x),
                [(True, _("Yes")), (False, _("No"))],
            ),
            # "efrom": (_("From"), lambda x: Q(efrom__icontains=x), "input"),
            "eto": (_("To"), lambda x: Q(eto__icontains=x), "input"),
            "retries": (_("Retries"), lambda x: Q(retries=x), "input"),
            "pk": (_("ID"), lambda x: Q(pk=x), "input"),
            "bounces_total_count": (
                _("Bounces"),
                SearchFilters.number("bounces_total_count"),
                "input",
            ),
            "bounces_soft_count": (
                _("Soft bounces"),
                SearchFilters.number("bounces_soft_count"),
                "input",
            ),
            "bounces_hard_count": (
                _("Hard bounces"),
                SearchFilters.number("bounces_hard_count"),
                "input",
            ),
        }


class EmailMessageCreate(GenCreate):
    model = EmailMessage
    form_class = EmailMessageForm


class EmailMessageCreateModal(GenCreateModal, EmailMessageCreate):
    pass


class EmailMessageDetails(GenDetail):
    model = EmailMessage
    groups = EmailMessageForm.__groups_details__()
    show_details = True
    tabs = [
        {
            "id": "receiveds",
            "name": _("Received"),
            "ws": "CDNX_emails_emailreceiveds_sublist",
            "rows": "base",
        }
    ]


class EmailMessageDetailsModal(EmailMessageDetails, GenDetailModal):
    pass


class EmailMessageUpdate(GenUpdate):
    model = EmailMessage
    form_class = EmailMessageForm
    show_details = True


class EmailMessageUpdateModal(GenUpdateModal, EmailMessageUpdate):
    pass


class EmailMessageDelete(GenDelete):
    model = EmailMessage


# ############################################
# EmailReceived
class EmailReceivedList(GenList):
    model = EmailReceived
    show_details = True
    search_filter_button = True
    datetime_filter = "updated"
    default_ordering = ["-created"]
    readonly = True
    gentranslate = {
        "sending": _("Sending"),
        "sent": _("Sent"),
        "notsent": _("Not sent!"),
        "waiting": _("Waiting"),
    }
    extra_context = {
        "menu": ["codenerix_email", "emailreceiveds"],
        "bread": [_("Emails"), _("Email Messages")],
    }


class EmailReceivedDetails(GenDetail):
    model = EmailReceived
    groups = EmailReceivedForm.__groups_details__()
    show_details = True
    readonly = True


class EmailReceivedDetailsModal(EmailReceivedDetails, GenDetailModal):
    pass


class EmailReceivedSubList(EmailReceivedList):
    def __limitQ__(self, info):  # noqa: N802
        limit = {}
        pk = info.kwargs.get("pk", None)
        limit["receiveds"] = Q(email__pk=pk)
        return limit
